/**
 * @license jsonapi
 * MIT license
 */

import { Injectable, NgModule } from '@angular/core';
import 'rxjs/add/operator/do';
import { camelCase, findIndex, forOwn, isArray, isObject, map, snakeCase } from 'lodash';
import { HTTP_INTERCEPTORS, HttpClient, HttpClientModule, HttpErrorResponse, HttpParams, HttpResponse } from '@angular/common/http';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/catch';
import { Observable as Observable$1 } from 'rxjs/Observable';
import 'rxjs/add/observable/of';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * `ApiConfig` represents the configuration of the api.
 */
class ApiConfig {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Libraries
// Config
class ApiInterceptor {
    /**
     * @param {?} config
     */
    constructor(config) {
        this.config = config;
    }
    /**
     * Intercept http request to add api url path.
     *
     * @param {?} req Http request
     * @param {?} next Http handler
     * @return {?}
     */
    intercept(req, next) {
        let /** @type {?} */ headers = req.headers;
        // Check headers
        if (this.config.headers) {
            forOwn(this.config.headers, (value, key) => {
                headers = headers.set(key, value);
            });
        }
        // Change api url
        const /** @type {?} */ apiReq = req.clone({
            headers,
            url: this.getApiUrl() + req.url,
        });
        // Pass on the cloned request instead of the original request.
        return next.handle(apiReq);
    }
    /**
     * Generate api url with version.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com
     * @return {?}
     */
    getApiUrl() {
        return this.config.url;
    }
}
ApiInterceptor.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ApiInterceptor.ctorParameters = () => [
    { type: ApiConfig, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Libraries
/**
 * `ResponseConverter` convert backend response to camelCase.
 *
 * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
 */
class ResponseConverter {
    /**
     * Intercept http request to convert backend response to camelCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof ResponseConverter
     * @param {?} req Request headers
     * @param {?} next Request handler
     * @return {?}
     */
    intercept(req, next) {
        return next.handle(req).do((event) => {
            if (event instanceof HttpResponse && event.body) {
                if (event.body.data) {
                    event.body.data = this.convert(event.body.data);
                }
                if (event.body.meta) {
                    event.body.meta = this.convert(event.body.meta);
                }
            }
        }, (event) => {
            if (event instanceof HttpErrorResponse) {
                if (event.error.errors) {
                    event.error.errors = this.convert(event.error.errors);
                }
            }
        });
    }
    /**
     * Convert object or array to camelCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof ResponseConverter
     * @param {?} data Object/array to convert
     * @return {?} Result converted
     */
    convert(data) {
        // Check if data is array, convert each item
        if (isArray(data)) {
            const /** @type {?} */ response = [];
            // Convert each item
            data.forEach((item) => {
                response.push(this.convert(item));
            });
            // Return new reponse
            return response;
        }
        // Check if data is a object, convert each key to camel case and check if value is an array or not
        if (isObject(data)) {
            const /** @type {?} */ response = {};
            // Convert each property key and value
            forOwn(data, (value, key) => {
                response[this.convertKey(key)] = this.convert(value);
            });
            // Return new response
            return response;
        }
        return data;
    }
    /**
     * Convert string to camelCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof ResponseConverter
     * @param {?} key String to convert
     * @return {?} String converted
     */
    convertKey(key) {
        const /** @type {?} */ keySplitted = key.split('.');
        return map(keySplitted, (k) => camelCase('' + k)).join('.');
    }
}
ResponseConverter.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ResponseConverter.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Libraries
/**
 * `RequestConverter` convert backend request body to snakeCase.
 *
 * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
 */
class RequestConverter {
    /**
     * Intercept http request to convert backend request body to snakeCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof RequestConverter
     * @param {?} req Request headers
     * @param {?} next Request handler
     * @return {?} Response
     */
    intercept(req, next) {
        let /** @type {?} */ reqBody = req.body;
        let /** @type {?} */ params = req.params;
        let /** @type {?} */ newReq;
        // Check if request body is not empty
        if (reqBody) {
            reqBody = this.convert(reqBody);
        }
        // Check query params
        if (params) {
            let /** @type {?} */ newParams = new HttpParams();
            // Parse each param key
            params.keys().map((key) => {
                newParams = newParams.set(this.convertKey(key), '' + params.get(key));
            });
            // Set new params
            params = newParams;
        }
        // Clone request
        newReq = req.clone({ body: reqBody, params });
        // Make request
        return next.handle(newReq);
    }
    /**
     * Convert object or array keys to snakeCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof RequestConverter
     * @param {?} data Object/array to convert
     * @return {?} Result converted
     */
    convert(data) {
        // Check if data is array, convert each item
        if (isArray(data)) {
            const /** @type {?} */ response = [];
            // Convert each item
            data.forEach((item) => {
                response.push(this.convert(item));
            });
            // Return new reponse
            return response;
        }
        // Check if data is a object, convert each key to snake case and check if value is an array or not
        if (isObject(data)) {
            const /** @type {?} */ response = {};
            // Convert each property key and value
            forOwn(data, (value, key) => {
                response[this.convertKey(key)] = this.convert(value);
            });
            // Return new response
            return response;
        }
        return data;
    }
    /**
     * Convert string to snakeCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof RequestConverter
     * @param {?} key String to convert
     * @return {?} String converted
     */
    convertKey(key) {
        const /** @type {?} */ keyConverted = key.indexOf('_') >= 0 ? key : snakeCase('' + key);
        return key.endsWith('[]') ? keyConverted + '[]' : keyConverted;
    }
}
RequestConverter.decorators = [
    { type: Injectable },
];
/** @nocollapse */
RequestConverter.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Libraries
/**
 * `HttpCacheService` service to cache requests.
 *
 * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
 * @export
 */
class HttpCacheService {
    constructor() {
        this.requests = [];
        this.responses = [];
    }
    /**
     * Returns a cached response, if any, or null if not present.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof HttpCacheService
     * @param {?} req
     * @return {?}
     */
    get(req) {
        const /** @type {?} */ reqIndex = this.findRequest(req);
        let /** @type {?} */ resp = null;
        // Find request and response
        if (reqIndex >= 0) {
            resp = this.responses[reqIndex];
        }
        return resp;
    }
    /**
     * Adds or updates the response in the cache.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof HttpCacheService
     * @param {?} req Request
     * @param {?} resp Response
     * @return {?}
     */
    put(req, resp) {
        // Push data
        this.requests.push(req);
        this.responses.push(resp);
    }
    /**
     * Find saved http request.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof HttpCacheService
     * @param {?} req Request to compare
     * @return {?} Response index
     */
    findRequest(req) {
        // Find http request
        return findIndex(this.requests, (savedReq) => {
            return savedReq.url === req.url;
        });
    }
}
HttpCacheService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
HttpCacheService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Libraries
// Services
const CACHE_KEY = 'cache';
/**
 * `CachingInterceptor` check cache param and cache request.
 *
 * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
 */
class CachingInterceptor {
    /**
     * Creates an instance of CachingInterceptor.
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof CachingInterceptor
     * @param {?} cache Service to cache request
     */
    constructor(cache) {
        this.cache = cache;
    }
    /**
     * Intercept http request to convert backend request body to snakeCase.
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof CachingInterceptor
     * @param {?} req Request headers
     * @param {?} next Request handler
     * @return {?} Response
     */
    intercept(req, next) {
        // Before doing anything, it's important to only cache GET requests.
        // Skip this interceptor if the request method isn't GET.
        if (req.method !== 'GET' || !this.cacheRequest(req)) {
            return next.handle(req);
        }
        // First, check the cache to see if this request exists.
        const /** @type {?} */ cachedResponse = this.cache.get(req);
        if (cachedResponse) {
            // A cached response exists. Serve it instead of forwarding
            // the request to the next handler.
            return Observable$1.of(cachedResponse);
        }
        // Delete cache key params
        const /** @type {?} */ newReq = req.clone({ params: req.params.delete(CACHE_KEY) });
        // No cached response exists. Go to the network, and cache
        // the response when it arrives.
        return next.handle(newReq).do((event) => {
            // Remember, there may be other events besides just the response.
            if (event instanceof HttpResponse) {
                // Update the cache.
                this.cache.put(newReq, event);
            }
        });
    }
    /**
     * Check if request should be cached
     *
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * \@memberof CachingInterceptor
     * @param {?} req Request
     * @return {?} True if should be cached
     */
    cacheRequest(req) {
        return req.params && req.params.get(CACHE_KEY) === 'true';
    }
}
CachingInterceptor.decorators = [
    { type: Injectable },
];
/** @nocollapse */
CachingInterceptor.ctorParameters = () => [
    { type: HttpCacheService, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Libraries
// Interfaces
/**
 * `BaseService` implements the basic REST interaction of any service with the api.
 *
 * \@typeparam R Represents the resource to be handled.
 *
 * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
 * @abstract
 */
class BaseService {
    /**
     * Creates a new `BaseService` instance.
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * @param {?} httpService Http service to make requests.
     * @param {?} config
     * @param {?} rName Name of resource.
     */
    constructor(httpService, config, rName) {
        this.httpService = httpService;
        this.config = config;
        this.rName = rName;
        this.resourceName = rName;
    }
    /**
     * Get a list of `R` objects
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * @param {?=} options Optional parameters
     * @return {?} An promise with an array of `R` objects.
     *
     */
    get(options) {
        return this.httpService.get(this.getPath(), options);
    }
    /**
     * Get one `R` object
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * @param {?} id Resource id
     * @param {?=} options
     * @return {?} An promise with an array of `R` objects.
     *
     */
    getOne(id, options) {
        return this.httpService.get(this.getPath(id), options);
    }
    /**
     * Create one `R` object.
     * @param {?} data Resource data data
     * @param {?=} options
     * @return {?}
     */
    post(data, options) {
        return this.httpService.post(this.getPath(), data, options);
    }
    /**
     * Update one `R` object.
     * @param {?} data Resource data data
     * @param {?=} options
     * @return {?}
     */
    put(data, options) {
        return this.httpService.put(this.getPath(data.id), data, options);
    }
    /**
     * Delete on `R` object.
     * @param {?} data Resource data data
     * @param {?=} options
     * @return {?}
     */
    delete(data, options) {
        return this.httpService.delete(this.getPath(data.id), options);
    }
    /**
     * Get current path
     * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
     * @param {?=} id
     * @return {?} String with current path.
     *
     */
    getPath(id) {
        let /** @type {?} */ path = '';
        const /** @type {?} */ prefix = this.config && this.config.prefix ? this.config.prefix : '';
        // Add prefix to api path
        if (prefix !== '') {
            path += '/' + prefix;
        }
        // Add version to api path
        const /** @type {?} */ version = this.config && this.config.version ? this.config.version : 0;
        if (version > 0) {
            path += '/v' + version;
        }
        // Add resource name
        path += '/' + this.resourceName;
        // Check if should add id
        return id && id > 0 ? path + '/' + id : path;
    }
}
BaseService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
BaseService.ctorParameters = () => [
    { type: HttpClient, },
    { type: ApiConfig, },
    null,
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Libraries
// Config
// Interceptors
/**
 * `JsonapiModule`
 *
 * @author Joaquin Volpe <joaquin.volpe\@woopasoft.com>
 */
class JsonapiModule {
    /**
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: JsonapiModule,
            providers: [
                HttpCacheService,
                {
                    provide: ApiConfig,
                    useValue: config
                },
                { provide: HTTP_INTERCEPTORS, useClass: ApiInterceptor, multi: true },
                { provide: HTTP_INTERCEPTORS, useClass: ResponseConverter, multi: true },
                { provide: HTTP_INTERCEPTORS, useClass: RequestConverter, multi: true },
                { provide: HTTP_INTERCEPTORS, useClass: CachingInterceptor, multi: true }
            ]
        };
    }
}
JsonapiModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [
                    HttpClientModule
                ]
            },] },
];
/** @nocollapse */
JsonapiModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Angular library starter
 * Build an Angular library compatible with AoT compilation & Tree shaking
 * Copyright Roberto Simonetti
 * MIT license
 * https://github.com/robisim74/jsonapi
 */
/**
 * Entry point for all public APIs of the package.
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { ApiConfig, ApiInterceptor, ResponseConverter, RequestConverter, CACHE_KEY, CachingInterceptor, BaseService, HttpCacheService, JsonapiModule };
//# sourceMappingURL=jsonapi.js.map
