import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';
import { IBaseModel, IResourceResponse, IResourcesResponse, ApiConfig } from '../jsonapi';
/**
 * `BaseService` implements the basic REST interaction of any service with the api.
 *
 * @typeparam R Represents the resource to be handled.
 *
 * @author Joaquin Volpe <joaquin.volpe@woopasoft.com>
 */
export declare abstract class BaseService<R extends IBaseModel> {
    protected httpService: HttpClient;
    protected config: ApiConfig;
    protected rName: string;
    /**
     * Resource name.
     */
    protected resourceName: string;
    /**
     * Creates a new `BaseService` instance.
     * @param httpService Http service to make requests.
     * @param rName Name of resource.
     * @param rNameSingular Singular name of resource.
     *
     * @author Joaquin Volpe <joaquin.volpe@woopasoft.com>
     */
    constructor(httpService: HttpClient, config: ApiConfig, rName: string);
    /**
     * Get a list of `R` objects
     * @param options Optional parameters
     * @returns An promise with an array of `R` objects.
     *
     * @author Joaquin Volpe <joaquin.volpe@woopasoft.com>
     */
    get(options?: {
        headers?: HttpHeaders;
        observe?: 'body';
        params?: HttpParams;
        reportProgress?: boolean;
        responseType?: 'json';
        withCredentials?: boolean;
    }): Observable<IResourcesResponse<R>>;
    /**
     * Get one `R` object
     * @param id Resource id
     * @returns An promise with an array of `R` objects.
     *
     * @author Joaquin Volpe <joaquin.volpe@woopasoft.com>
     */
    getOne(id: number, options?: {
        headers?: HttpHeaders;
        observe?: 'body';
        params?: HttpParams;
        reportProgress?: boolean;
        responseType?: 'json';
        withCredentials?: boolean;
    }): Observable<IResourceResponse<R>>;
    /**
     * Create one `R` object.
     * @param data Resource data data
     */
    post(data: R, options?: {
        headers?: HttpHeaders;
        observe?: 'body';
        params?: HttpParams;
        reportProgress?: boolean;
        responseType?: 'json';
        withCredentials?: boolean;
    }): Observable<IResourceResponse<R>>;
    /**
     * Update one `R` object.
     * @param data Resource data data
     */
    put(data: R, options?: {
        headers?: HttpHeaders;
        observe?: 'body';
        params?: HttpParams;
        reportProgress?: boolean;
        responseType?: 'json';
        withCredentials?: boolean;
    }): Observable<IResourceResponse<R>>;
    /**
     * Delete on `R` object.
     * @param data Resource data data
     */
    delete(data: R, options?: {
        headers?: HttpHeaders;
        observe?: 'body';
        params?: HttpParams;
        reportProgress?: boolean;
        responseType?: 'json';
        withCredentials?: boolean;
    }): Observable<IResourceResponse<R>>;
    /**
     * Get current path
     * @param id? Resource id
     * @returns String with current path.
     *
     * @author Joaquin Volpe <joaquin.volpe@woopasoft.com>
     */
    protected getPath(id?: number): string;
}
